
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "../../src/initng.h"


#include <stdio.h>
#include <string.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"

#include <assert.h>

#include "initng_pause.h"

s_entry U_U_DELAY = { "start_u_pause", INT, 13 };
s_entry U_DELAY = { "start_pause", INT, 11 };
s_entry D_U_DELAY = { "stop_u_pause", INT, 12 };
s_entry D_DELAY = { "stop_pause", INT, 10 };

int do_pause(active_h * s, process_h * p)
{
    int t = 0;

    assert(s);
    assert(p);

    D_(" %s ", s->name);

    if (p->pt == T_START)
    {
        /* delay execution a bit */
        if ((t = active_db_get_int(&U_U_DELAY, s)) > 0)
        {
            D_("Sleeping for %i miliseconds.\n", t);
            usleep(t);
        }
        else if ((t = active_db_get_int(&U_DELAY, s)) > 0)
        {
            D_("Sleeping for %i seconds.\n", t);
            sleep(t);
        }
        return (TRUE);
    }
    else if (p->pt == T_STOP)
    {
        /* delay execution a bit */
        if ((t = active_db_get_int(&D_U_DELAY, s)) > 0)
        {
            D_("Sleeping for %i miliseconds.\n", t);
            usleep(t);
        }
        else if ((t = active_db_get_int(&D_DELAY, s)) > 0)
        {
            D_("Sleeping for %i seconds.\n", t);
            sleep(t);
        }
        return (TRUE);
    }
    F_("Dont know hot to sleep this process type (%s)!\n", s->name);
    return (FALSE);
}

int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }
    initng_sdt_add(&U_U_DELAY);
    initng_sdt_add(&U_DELAY);
    initng_sdt_add(&D_U_DELAY);
    initng_sdt_add(&D_DELAY);
    return (initng_add_hook(A_FORK, 90, &do_pause));
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&U_U_DELAY);
    initng_sdt_del(&U_DELAY);
    initng_sdt_del(&D_U_DELAY);
    initng_sdt_del(&D_DELAY);
    initng_del_hook(A_FORK, &do_pause);
}
