/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <assert.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"

#include "initng_stdout.h"

s_entry STDOUT = { "stdout", STRING, 6 };

#define MAX_P_ENTRY 200
local_plugin_database_h **local_database;


/* Returns pointer to old data if possible, and alloc's a new spot if needed.
   Note that this uses strdup on service->name, since service->name might be free'd without us knowing about it.
 */
local_plugin_database_h *find_service(active_h * service)
{
    /* Scans through our local database, returns a new one if we cant find an old one. */
    int i;
    int foundfree = 0;

    assert(service);
    assert(service->name);

    S_;

    D_("Scan for service %s\n", service->name);
    for (i = 0; i < MAX_P_ENTRY; i++)
        if (local_database[i])
        {
            if (strcmp(local_database[i]->servicename, service->name) == 0)
                return local_database[i];
        }
        else
            foundfree = i;

    /* No free service found */
    if (foundfree == 0)
    {
        F_("Service database full!");
        return (FALSE);
    }
    D_("Allocating new service\n");
    local_database[foundfree] =
        (local_plugin_database_h *) initng_calloc(1,
                                                  sizeof
                                                  (local_plugin_database_h));
    local_database[foundfree]->servicename = i_strdup(service->name);
    local_database[foundfree]->fd = 0;
    local_database[foundfree]->append = FALSE;
    return local_database[foundfree];
}

void program_output(active_h * service, int datalen, process_h * x,
                    int forceflush)
{
    char *filename = NULL;
    assert(service);
    assert(service->name);
    assert(x);

    D_("service: \"%s\" have something to say.\n", service->name);

#ifdef DEBUG
    if (service->start_process == x)
        D_("start process fd: #%i\n", x->out_pipe[0]);
    else if (service->stop_process == x)
        D_("stop process fd: #%i\n", x->out_pipe[0]);

#endif
    /* Check if this service has keyword STDOUT */
    if (!active_db_is(&STDOUT, service))
    {
        D_("STDOUT not set.\n");
        return;
    }

    /* Get a new spot for entry of the data */
    local_plugin_database_h *entry = find_service(service);

    if (entry == NULL)
        return;

    if (datalen < 3)
        return;

    /* logfile file descriptor is 0, open it */
    if (entry->fd == 0)
    {
        filename =
            fix_variables(active_db_get_string(&STDOUT, service), service);
        /* If entry->append then open with O_APPEND, else truncate */
        if (!entry->append)
            entry->fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC);
        else
            entry->fd = open(filename, O_WRONLY | O_CREAT | O_APPEND);
        if (entry->fd < 1)
        {
            entry->fd = 0;
            F_("Error opening %s, err : %s\n", filename, strerror(errno));
        }
    }
    /* Write data to logfile */
    D_("Writing data...\n");
    if (write(entry->fd, &x->buffer[x->buffer_pos], datalen) != datalen)
        F_("Error writing to %s's log, err : %s\n", service->name,
           strerror(errno));
    free(filename);

    if (forceflush)                         /* If forceflush, this is probably the last this service is writing to stdout, close file (or buffer full) */
    {
        if (entry->fd > 0)
            close(entry->fd);
        /* Set append flag, in case buffer is full / in case we wanna log service->stop_process aswell, we dont want to truncate the log */
        entry->append = TRUE;
        entry->fd = 0;
    }
}

int module_init(const char *version)
{
    int i;

    S_;

    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&STDOUT);

    /* This is quite bad and should be changed... */
    local_database =
        initng_calloc(MAX_P_ENTRY + 1, sizeof(local_plugin_database_h));
    for (i = 0; i < MAX_P_ENTRY; i++)
        local_database[i] = NULL;


    initng_add_hook(PIPEWATCHERS, 30, &program_output);
    return (TRUE);
}

void module_unload(void)
{
    int i;

    S_;

    D_("module_unload();\n");

    initng_sdt_del(&STDOUT);

    initng_del_hook(PIPEWATCHERS, &program_output);

    if (!local_database)
        return;


    for (i = 0; i < MAX_P_ENTRY; i++)
        if (local_database[i])
        {
            /* Remember to free servicename, as it's strdup'ed of service->name, not a pointer */
            if (local_database[i]->servicename)
                free(local_database[i]->servicename);
            if (local_database[i]->fd > 0)
                close(local_database[i]->fd);
            free(local_database[i]);
        }

    free(local_database);
    local_database = NULL;

}
