/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef INITNG_CONTROL_COMMAND
#define INITNG_CONTROL_COMMAND
#include <stdarg.h>
#include "initng_list.h"

typedef enum
{
    COMMAND_FAIL = 0,
    DATA_COMMAND  = 1,
    VOID_COMMAND = 2,
    INT_COMMAND = 3,
    TRUE_OR_FALSE_COMMAND = 4,
    STRING_COMMAND = 5,
    PRINT_COMMAND = 6
} e_com_type;

/*
   typedef union
   {
   void (*void_command_call) (char *data);
   int (*int_command_call) (char *data);
   char *(*string_command_call) (char *data);
   } u_command_call;
 */
typedef enum
{
    NO_OPT = 0,
    USES_OPT = 1,
    REQUIRES_OPT = 2
} e_opt_type;

typedef struct
{
    char command_id;
    const char *long_id;
    e_com_type com_type;
    e_opt_type opt_type;
    union
    {
        void (*void_command_call) (char *data);
        int (*int_command_call) (char *data);
        char *(*string_command_call) (char *data);
        void (*print_command_call) (char *data, FILE * fd);
        void (*void_command_void_call) (void);
        int (*int_command_void_call) (void);
        char *(*string_command_void_call) (void);
        void (*print_command_void_call) (FILE * fd);
    } u;
    const char *description;

    struct list_head list;
} s_command;

int initng_command_add(s_command * cmd);
void initng_command_free_all(void);
s_command *initng_command_find_by_command_id(char cid);
s_command *initng_command_find_by_command_string(char *name);

#define initng_command_del(cmd) list_del(&(cmd)->list)
#define while_command_db(current) list_for_each_entry_prev(current, &g.command_db.list, list)
#define while_command_db_safe(current, safe) list_for_each_entry_prev_safe(current, safe, &g.command_db.list, list)

#endif
